package client;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.nio.ByteBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.concurrent.ConcurrentHashMap;

import net.AppServerSocket;
import net.AppSocket;
import net.ByteOp;
import net.ConnectionInfo;
import net.TimeoutLimitException;
import net.Transport;
import server.LoginException;
import server.RepositorioUsers;
import server.UnnavailableLoginException;
import server.User;

public class Client extends Thread{
	ServerConnection server;
	private boolean logado;
	private boolean conectado;
	private boolean vive;
	private FriendThread ft;
	private AppServerSocket requestSocket;
	private int nextId;
	private ConcurrentHashMap<InetAddress, AppSocket> tabelaDeSockets;
	private ConcurrentHashMap<InetAddress, ChatThread> listaDeThreads;
	private Transport trans;
	private ClientGuiListener listener;
	public Client(ServerConnection server, Transport t) throws IOException, ClassNotFoundException{
		conectado = false;
		logado = false;
		vive = true;
		this.server = server;
		ft = new FriendThread(this);
		requestSocket = new AppServerSocket(t,0);
		tabelaDeSockets = new ConcurrentHashMap<InetAddress, AppSocket>();
		listaDeThreads = new ConcurrentHashMap<InetAddress, ChatThread>();
		nextId = 2;
		this.trans = t;
	}
	public void addGuiListener(ClientGuiListener listener){
		this.listener = listener;
		ft.addListener(listener);
	}
	public void cadastrar(User usuario) throws IOException, TimeoutLimitException, UnnavailableLoginException, ServerException{
		this.connect();
		server.cadastrar(usuario);
	}
	public void connect() throws TimeoutLimitException{
	
		if(!conectado){

			server.connect();
			
			conectado = true;
		}

	}
	public Hashtable<String, ConnectionInfo> getFriendList() throws TimeoutLimitException, IOException, ClassNotFoundException{
		return server.getOnlineFriends();
	}
	public void logar(String login, String senha) throws TimeoutLimitException, LoginException{
		if(!logado){
			if(!conectado){
				this.connect();
			}
			server.logar(login, senha);
			logado = true;
			ft.start();
		}
	}
	public ArrayList<String> getAmigos() throws TimeoutLimitException, IOException, ClassNotFoundException{
		return server.getFriends();
	}
	public void adicionarAmigo(String login) throws LoginException, ServerException, TimeoutLimitException, IOException, UnnavailableLoginException{
		server.addFriend(login);
		
	}

	public void logout() throws TimeoutLimitException{
		server.logout();
	}
	public void setOffline(){
		this.logado = false;
		this.conectado = false;
	}
	public ClientGuiListener getListener(){
		return this.listener;
	}
	public void run(){
		AppSocket sock = null;
		while(vive){
			ConnectionInfo ci = requestSocket.receiveRequest();
			System.out.println("Recebeu requisio de: "+ci.toString());
			if(!tabelaDeSockets.containsKey(ci.getIp())){
				System.out.println("No estava na tabela");
				sock = new AppSocket(trans, nextId);
				nextId++;
				sock.match(ci);
				tabelaDeSockets.put(ci.getIp(), sock);
				ChatThread thread = new ChatThread(this, sock, ci);
				listaDeThreads.put(ci.getIp(), thread);
				listener.connectionReceived(thread, ci);
			}else{
		
			}
			if(sock != null){
				sock.acceptConnection();
			}
		}
	}
	public void sendMsg(String msg, ConnectionInfo ci, ClientGuiListener listener) throws TimeoutLimitException{
		byte[] rawMsg = null;
		try {
			rawMsg = msg.getBytes("UTF-16");
		} catch (UnsupportedEncodingException e) {
			rawMsg = msg.getBytes();
			System.out.println("Charset no suportado");
		} 
		AppSocket sock = this.connectToClient(ci, listener);
		ByteBuffer pacote = ByteBuffer.allocate(rawMsg.length+5);
		pacote.put((byte)0);
		pacote.put(ByteOp.intToByte(rawMsg.length));
		pacote.put(rawMsg);
		
		pacote.flip();
		if(sock == null){
		
			sock = tabelaDeSockets.get(ci.getIp());
		}
		System.out.println("-------------"+rawMsg.length);
		sock.send(pacote.array());
	}
	private synchronized AppSocket connectToClient(ConnectionInfo ci, ClientGuiListener listener) throws TimeoutLimitException{
		AppSocket retorno = null;
		if(!tabelaDeSockets.containsKey(ci.getIp())){
			AppSocket sock = new AppSocket(trans, nextId);
			ChatThread thread = new ChatThread(this, sock, ci);
			listaDeThreads.put(ci.getIp(), thread);
			thread.addListener(listener);
			nextId++;
			ci.setId(0);
			sock.connect(ci);
			tabelaDeSockets.put(ci.getIp(), sock);
			retorno = sock;
		}
		return retorno;
	}
}
