package server;
import java.io.IOException;
import java.net.InetAddress;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;
import net.*;
public class Server extends Thread{

	private Transport transport;
	private AppServerSocket mainSocket;
	private int nextId;
	private ServerListener guiListener;
	private CopyOnWriteArrayList<ConnectionInfo> peers;	
	
	private RepositorioUsers repositorio;
	private ConcurrentHashMap<String, ConnectionInfo> listaOnline;
	private Hashtable<InetAddress, ServerThread> tabelaThreads;
	public Server(RepositorioUsers repositorio, int porta) throws SocketException{
		transport = new Transport(porta);
		this.repositorio = repositorio;
		mainSocket = new AppServerSocket(transport, 0);
		AppSocket checkSocket = new AppSocket(transport, 1);
		transport.addListenerSocket(checkSocket);
		this.nextId = 2;
		peers = new CopyOnWriteArrayList<ConnectionInfo>();
		listaOnline = new ConcurrentHashMap<String, ConnectionInfo>();
		tabelaThreads = new Hashtable<InetAddress, ServerThread> ();
	}
	public ConcurrentHashMap<String, ConnectionInfo> getListaOnline(){
		return this.listaOnline;
	}
	
	public void addListener(ServerListener guiListener){
		this.guiListener = guiListener;
	}
	public CopyOnWriteArrayList<ConnectionInfo> getPeers(){
		return this.peers;
	}

	public ServerListener getGuiListener(){
		return guiListener;
	}
	public void run(){
		AppSocket sock = null;
		while(true){
			ConnectionInfo coisa = mainSocket.receiveRequest();
			if(!peers.contains(coisa)){
				System.out.println("No estava na lista");
				peers.add(coisa);
				sock = new AppSocket(transport,nextId);
				nextId++;
				sock.match(coisa);
				ServerThread thread = new ServerThread(this, sock, coisa);
				tabelaThreads.put(coisa.getIp(), thread);
				thread.start();
			}else{
				ServerThread temp = tabelaThreads.get(coisa.getIp());
				if(temp != null){
					temp.kill();
					tabelaThreads.remove(coisa.getIp());
				}
				sock = new AppSocket(transport,nextId);
				nextId++;
				sock.match(coisa);
				ServerThread thread = new ServerThread(this, sock, coisa);
				thread.start();
				tabelaThreads.put(coisa.getIp(), thread);
				peers.add(coisa);
			}
			guiListener.userConnected(coisa.getIp().getHostAddress());
			sock.acceptConnection();
			
		}
	}
	public void removeMe(ServerThread st){
		listaOnline.remove(st);
	}
	public ArrayList<String> getFriendsList(String login) throws UsuarioNaoExistenteException{
		return repositorio.buscar(login).getContatos();
	}
	public void addUser(String login, String senha,
			String email, String twitter) throws UnnavailableLoginException, IOException {
		
			User a = new User( login, senha, email, twitter, login, null);
			repositorio.adicionar(a);
	}

	public void login(String login, String senha, ConnectionInfo ci) throws LoginException, UsuarioNaoExistenteException {
		User temp = repositorio.buscar(login);
		if(temp != null){
			
			if(temp.getSenha().equals(senha)){
				if(!listaOnline.containsKey(login)){
					listaOnline.put(login, ci);
				}else{
					throw new LoginException();
				}
			}else{
				throw new LoginException();
			}
		}else{
			throw new LoginException();
		}
	}
	
	public void logout(String login) throws LoginException, UsuarioNaoExistenteException {

			if(listaOnline.containsKey(login)){
				listaOnline.remove(login);
			}else{
				throw new LoginException();
			}
			
	}
	
	
	public void changeUserName(String login, String novoNome)
			throws UsuarioNaoExistenteException, IOException {
		repositorio.editarNick(login, novoNome);
	}

	public void changeStatus(String login, String novoStatus)
			throws UsuarioNaoExistenteException, IOException {
		repositorio.editarStatus(login, novoStatus);
	}

	public void changePassword(String login, String novaSenha)
			throws UsuarioNaoExistenteException, IOException {
		repositorio.editarSenha(login, novaSenha);
	}

	public void changeTwitter(String login, String novoTwitter)
			throws UsuarioNaoExistenteException, IOException {
		repositorio.editarTwitter(login, novoTwitter);
	}

	public void changeEmail(String login, String novoEmail)
			throws UsuarioNaoExistenteException, IOException {
		repositorio.editarEmail(login, novoEmail);
	}
	
	public void addFriend(String login, String novoAmigo)
	throws UsuarioNaoExistenteException, IOException {
		repositorio.adicionarAmigo(login, novoAmigo);
	}
	
	public void removeFriend(String login, String exAmigo)
	throws UsuarioNaoExistenteException, IOException {
		repositorio.removerAmigo(login, exAmigo);
	}
	
	public ArrayList<String> getContatos(String login)
	throws UsuarioNaoExistenteException, IOException {
		return repositorio.getContatos(login);
	}
}
